//--------------------------------------------------------------------
//  macphy.c
//
//  Copyright (c) 2005 National Semiconductor Corporation.
//  All Rights Reserved.
//
//  QNX6 Network Device Driver for National Semiconductor
//  DP83816 "MacPhyter" 10/100 Ethernet Controller.
//
//  io-net initialization & configuration and several io-net entry points
//  live in this file. The main event task is also in this file as well
//  as some utility functions.
//--------------------------------------------------------------------

#ifdef __GNUC__
#define UNUSED __attribute__ ((unused))
#endif
static char
CvsId[] UNUSED = "FIXME";

#include <macphy.h>

#define NIC_INTERRUPT_EVENT		0
#define NIC_TIMER_EVENT			1

io_net_dll_entry_t io_net_dll_entry = {
		2, 
		macphy_init, 
		NULL };

io_net_registrant_funcs_t macphy_funcs = {
		8, 
		NULL,
		macphy_send_packets, 
		macphy_receive_complete,
		macphy_shutdown1,
		macphy_shutdown2,
		macphy_advertise,
		macphy_devctl, 
		macphy_flush, 
		NULL
		};


io_net_registrant_t macphy_entry = {
        _REG_PRODUCER_UP,
        "devn-macphy",
        "en",
        NULL,
        NULL,
        &macphy_funcs,
        0
	};


/*
 * local prototypes 
 */

static void macphy_halt( Nic_t *nic );
static void macphy_reset( Nic_t *nic );
static int macphy_read_mac( Nic_t *nic );
static int macphy_media( Nic_t *nic );
static int macphy_config( Nic_t *nic );
static int macphy_timer( Nic_t *nic );
static void macphy_process_interrupt( Nic_t *nic );
static void macphy_initialize( Nic_t *nic );
static int macphy_filter( Nic_t *nic );

void macphy_event_handler( Nic_t *nic );

/*******************************************************************************
 *
 * macphy_init()
 *
 * This is the driver's io-net entry point. This function detects, enumerates,
 * and initializes all MacPhyter network interaces.
 *
 * Parameters
 * ----------
 *		void *dll_hdl: Private handle used by io-net. Save this handle for
 *		calls up to io-net.
 *
 *		dispatch_t *dpp: Dispatch handle.
 *
 *		io_net_self_t *ion: Pointer to a data structure of io-net functions
 *		that this driver will call.
 *
 *		char *options: Options passed from io-net command line to this driver.
 *
 * Returns: -1 on error
 */
int
macphy_init(
	void *dll_hdl,
	dispatch_t *dpp,
	io_net_self_t *ion,
	char *options
)
{
	int ret;
	dpp = dpp;
    printf("hello world, macphy calling\n");
	// FIXME call into MPL 

	if( ret = macphy_detect( dll_hdl, ion, options ) ) {
        printf("macphy_detect returns %d\n",ret);
		errno = ret;
		return( -1 );
	}
    printf("macphy_init okay\n");
	return( 0 );
}

/*******************************************************************************
 *
 */
int
macphy_flush( int reg_hdl, void *hdl )
{
	Nic_t				*nic = (Nic_t *)hdl;
	Nic_Macphy_Ext_t	*ext;
	npkt_t				*npkt;
	npkt_t				*tmp;

	ext = (Nic_Macphy_Ext_t *)nic->ext;

	pthread_mutex_lock( &ext->mutex );
	npkt = ext->nhead;
	ext->nhead = ext->ntail = NULL;

	for( ; npkt; npkt = tmp ) {
		tmp = npkt->next;
		ion_tx_complete(ext->reg_hdl, npkt);
	}

	pthread_mutex_unlock( &ext->mutex );

	return( 0 );
}



/*==============================================================================
 *
 */
static int
macphy_media( Nic_t *nic )
{
	Nic_Macphy_Ext_t	*ext;
	int					iobase;

	ext		= (Nic_Macphy_Ext_t *)nic->ext;
	iobase	= ext->iobase;

	return( EOK );
}


/*******************************************************************************
 *
 * io-net entry point
 * XXX advertise driver's capabilities.
 *
 */
int
macphy_advertise(int reg_hdl, void *func_hdl)
{
npkt_t						*npkt;
net_buf_t					*nb;
net_iov_t					*iov;
Nic_t						*nic;
Nic_Macphy_Ext_t			*ext;
io_net_msg_dl_advert_t		*ap;

	nic = (Nic_t *)(func_hdl);
	ext = (Nic_Macphy_Ext_t *) nic->ext;

	if((npkt = ion_alloc_npkt(sizeof(*nb) + sizeof *iov, (void **)&nb)) == NULL)
	{
		return(0);
	}

	if((ap = ion_alloc(sizeof(*ap), 0)) == NULL)
	{
		ion_free(npkt);
		return (0);
	}

	TAILQ_INSERT_HEAD(&npkt->buffers, nb, ptrs);

	iov = (net_iov_t *)(nb + 1);

	nb->niov = 1;
	nb->net_iov = iov;
	iov->iov_base = ap;
	iov->iov_len = sizeof *ap;

	memset(ap, 0x00, sizeof *ap);
	ap->type   = _IO_NET_MSG_DL_ADVERT;
	ap->iflags = (IFF_SIMPLEX | IFF_BROADCAST | IFF_RUNNING);
	if( nic->flags & NIC_FLAG_MULTICAST )
		ap->iflags |= IFF_MULTICAST;

	ap->mtu_min      = 0;
	ap->mtu_max      = nic->mtu;
	ap->mtu_preferred = nic->mtu;
	strcpy(ap->up_type, "en");
	itoa(nic->lan, ap->up_type + 2, 10);

	strcpy(ap->dl.sdl_data, ap->up_type);

	ap->dl.sdl_len = sizeof(struct sockaddr_dl);
	ap->dl.sdl_family = AF_LINK;
	ap->dl.sdl_index  = nic->lan;
	ap->dl.sdl_type = IFT_ETHER;
	ap->dl.sdl_nlen = strlen(ap->dl.sdl_data); /* not null terminated */
	ap->dl.sdl_alen = 6;
	memcpy(ap->dl.sdl_data + ap->dl.sdl_nlen, nic->current_address, 6);

	npkt->org_data = ap;
	npkt->flags |= _NPKT_MSG;
	npkt->iface = 0;
	npkt->framelen = sizeof *ap;
	npkt->tot_iov = 1;
	if(ion_add_done( ext->reg_hdl, npkt, nic ) == -1) {
		ion_free(ap);
		ion_free(npkt);
		return(0);
	}

	if(ion_rx_packets(ext->reg_hdl, npkt, 0, 0, ext->cell, nic->lan, 0) == 0) {
		ion_tx_complete(ext->reg_hdl, npkt);
	}

	return(0);
}

/*******************************************************************************
 *
 */
npkt_t
*macphy_alloc_npkt(Nic_t *nic, size_t size) 

{
Nic_Macphy_Ext_t	        *ext;
npkt_t                          *npkt;
net_buf_t                       *nb;
net_iov_t                       *iov;
char                            *ptr;

	ext = (Nic_Macphy_Ext_t *) nic->ext;

	if((npkt = ion_alloc_npkt(sizeof(net_buf_t) + sizeof(net_iov_t), 
							(void **) &nb)) == NULL) {
		return( NULL );
		}

	if((ptr = ion_alloc(size + 2, 0)) == NULL) {
		ion_free(npkt);
		return(NULL);
		}

	TAILQ_INSERT_HEAD(&npkt->buffers, nb, ptrs);

	iov = (net_iov_t *)(nb + 1);

	nb->niov                = 1;
	nb->net_iov             = iov;

	iov->iov_base   = (void *) (ptr + 2);
	iov->iov_len    = size;
	iov->iov_phys   = (paddr_t)(ion_mphys(iov->iov_base));

	npkt->org_data  = ptr;
	npkt->next      = NULL;
	npkt->tot_iov	= 1;
	return (npkt);
}

// fix for ENDIAN
int
macphy_calc_crc( char *buf, int len )
{
int				i;
int				j;
uchar_t			data;
const ulong_t	poly = 0xedb88320;
ulong_t			crc  = 0xffffffff;

	for( i = 0; i < len; i++ ) {
		data = buf[i];
		for( j = 0; j < 8; j++ ) {
			crc = ( crc >> 1 ) ^ ( ( ( crc ^ data ) & 0x01 ) ? poly : 0 );
			data >>= 1;
		}
	}
	return( crc );
}

/*==============================================================================
 *
 */
#if 0
static int
macphy_multicast( Nic_t *nic, mac_addr_t *mlist, int entries )

{
uint_t				cnt;
uint_t				bit;
int					iobase;
Nic_Macphy_Ext_t	*ext;
uchar_t				invert3[] = { 0, 4, 2, 6, 1, 5, 3, 7 };

	ext		= (Nic_Macphy_Ext_t *)nic->ext;
	iobase	= ext->iobase;

	pthread_mutex_lock( &ext->mutex );

	memset( ext->multicast_list, 0x00, 8 );

	for( cnt = 0; cnt < entries; cnt++ ) {
		bit = macphy_calc_crc( mlist[cnt], ETH_MAC_LEN ) & 0x3f;
		ext->multicast_hashed[ invert3[ bit & 7 ] ] |= ( 1 << invert3[ ( bit >> 3 ) & 7 ] );
	}

	SMC_SELECT_BANK( iobase, 3 );

// fix this is wrong!!!
	for( cnt = 0; cnt < 9; cnt++ ) {
		OUT8( iobase + SMC_MT0 + (cnt * SMC_ALIGN), ext->multicast_hashed[cnt] );
		}

	pthread_mutex_unlock( &ext->mutex );

	return( 0 );
}
#endif

/*==============================================================================
 *
 */
static int
macphy_filter( Nic_t *nic )
{
int					iobase;
Nic_Macphy_Ext_t	*ext;

	ext		= (Nic_Macphy_Ext_t *)nic->ext;
	iobase	= ext->iobase;

	pthread_mutex_lock( &ext->mutex );
	pthread_mutex_unlock( &ext->mutex );

	return( 0 );
}

/*==============================================================================
 *
 */
static void
macphy_initialize( Nic_t *nic )
{
Nic_Macphy_Ext_t	*ext;
int					iobase;

	ext		= (Nic_Macphy_Ext_t *)nic->ext;
	iobase	= ext->iobase;
    return;
}


/*==============================================================================
 *
 */
static void
macphy_halt( Nic_t *nic )
{
	int					iobase;
	Nic_Macphy_Ext_t	*ext;

	ext		= (Nic_Macphy_Ext_t *)nic->ext;
	iobase	= ext->iobase;

    return;
}

/*==============================================================================
 *
 */
static void
macphy_reset( Nic_t *nic )
{
	Nic_Macphy_Ext_t	*ext;

	ext = (Nic_Macphy_Ext_t *)nic->ext;

	ext->timer	= 0;
	macphy_halt( nic );
	macphy_initialize( nic );
}

/*==============================================================================
 *
 */
static int
macphy_read_mac( Nic_t *nic )
{
	Nic_Macphy_Ext_t	*ext;
	int					iobase;

	ext		= (Nic_Macphy_Ext_t *)nic->ext;
	iobase	= ext->iobase;

	return( 0 );
}

/*==============================================================================
 *
 */
static int
macphy_config( Nic_t *nic )
{
	Nic_Macphy_Ext_t	*ext;

	ext		= (Nic_Macphy_Ext_t *)nic->ext;
	return( 0 );
}

/*******************************************************************************
 *
 * XXX register driver with io-net
 * XXX called from detect.c/_scan().
 */
int
macphy_register_device( Nic_t *nic, io_net_self_t *ion, void *dll_hdl)
{
	Nic_Macphy_Ext_t		*ext;
	pthread_attr_t			pattr;
	pthread_mutexattr_t		mattr;
	struct sched_param		param;
	struct sigevent			event;
	struct itimerspec		value;
	uint16_t				lan;

	ext						= (Nic_Macphy_Ext_t *)nic->ext;
    ext->ion				= ion;
    ext->dll_hdl			= dll_hdl;
	macphy_entry.func_hdl	= (void *)nic;

 	if( !ext->max_pkts ) {
		ext->max_pkts = 100;
	}

	pthread_attr_init( &pattr );
	pthread_attr_setschedpolicy( &pattr, SCHED_RR );
	param.sched_priority = 21;
	pthread_attr_setschedparam( &pattr, &param );
	pthread_attr_setinheritsched( &pattr, PTHREAD_EXPLICIT_SCHED );

	if( ( ext->chid = ChannelCreate( _NTO_CHF_DISCONNECT | _NTO_CHF_UNBLOCK ) ) != -1 ) {
		if( ( ext->coid = ConnectAttach( 0, 0, ext->chid, _NTO_SIDE_CHANNEL, 0 ) ) != -1 ) {
			pthread_mutexattr_init( &mattr );
			mattr.flags = PTHREAD_RECURSIVE_ENABLE;
			if( pthread_mutex_init( &ext->mutex, &mattr ) != -1 ) {
					/* create the interface/interrupt/event thread */
				if( pthread_create( &ext->tid, &pattr, (void *)macphy_event_handler, nic ) == 0 ) {
					if( macphy_config( nic ) != -1 ) {
						event.sigev_notify		= SIGEV_PULSE;
						event.sigev_coid		= ext->coid;
						event.sigev_code		= NIC_INTERRUPT_EVENT;
						event.sigev_priority	= NIC_PRIORITY;
						if( ( ext->iid = InterruptAttachEvent( nic->cfg.IRQRegisters[0], &event, _NTO_INTR_FLAGS_TRK_MSK ) ) != -1 ) {
							event.sigev_notify		= SIGEV_PULSE;
							event.sigev_code		= NIC_TIMER_EVENT;
							event.sigev_coid		= ext->coid;
							event.sigev_priority	= NIC_PRIORITY;
							if( timer_create( CLOCK_REALTIME, &event, &ext->timerid ) != -1 ) {
								value.it_value.tv_sec	= value.it_interval.tv_sec	= 1;
								value.it_value.tv_nsec	= value.it_interval.tv_nsec	= 0;
								if( timer_settime( ext->timerid, 0, &value, NULL ) != -1 ) {
									if( ion_register( dll_hdl, &macphy_entry, &ext->reg_hdl, &ext->cell, &lan ) != -1 ) {
										nic->lan = lan;
							 			ext->ion->devctl( ext->reg_hdl, DCMD_IO_NET_MAX_QUEUE,
								                  &ext->max_pkts, sizeof( ext->max_pkts ), NULL );
										ext->acceptingTx = 1;
										return( 0 );
									}
									else {
										slogf( _SLOGC_NETWORK, _SLOG_ERROR, "devn-macphy:  ion_register" );
									}
								}
								else {
									slogf( _SLOGC_NETWORK, _SLOG_ERROR, "devn-macphy:  timer_settime" );
								}
								timer_delete( ext->timerid );
							}
							else {
								slogf( _SLOGC_NETWORK, _SLOG_ERROR, "devn-macphy:  timer_create" );
							}
							InterruptDetach( ext->iid );
						}
						else {
							slogf( _SLOGC_NETWORK, _SLOG_ERROR, "devn-macphy:  InterruptAttachEvent" );
						}
					}
					else {
						slogf( _SLOGC_NETWORK, _SLOG_ERROR, "devn-macphy:  macphy_config" );
					}
					pthread_kill( ext->tid, SIGKILL );
				}
				else {
					slogf( _SLOGC_NETWORK, _SLOG_ERROR, "devn-macphy:  pthread_create" );
				}
				pthread_mutex_destroy( &ext->mutex );
			}
			else {
				slogf( _SLOGC_NETWORK, _SLOG_ERROR, "devn-macphy:  pthread_mutex_init" );
			}
			ConnectDetach( ext->coid );
		}
		else {
			slogf( _SLOGC_NETWORK, _SLOG_ERROR, "devn-macphy:  ConnectAttach" );
		}
		ChannelDestroy( ext->chid );
	}
	else {
		slogf( _SLOGC_NETWORK, _SLOG_ERROR, "devn-macphy:  ChannelCreate" );
	}
	return( -1 );
}

/*==============================================================================
 *
 */
static int
macphy_timer( Nic_t *nic )
{
	Nic_Macphy_Ext_t	*ext;

	ext = (Nic_Macphy_Ext_t *)nic->ext;

	if( ext->timer ) {
		if( !( --ext->timer ) ) {
            pthread_mutex_lock( &ext->mutex );
			macphy_reset( nic );
            pthread_mutex_unlock( &ext->mutex );
			return( 0 );
		}
	}

	return( 0 );
}

/*==============================================================================
 *
 */
static void
macphy_process_interrupt( Nic_t *nic )
{
	Nic_Macphy_Ext_t	*ext;
	int					iobase;

	ext			= (Nic_Macphy_Ext_t *)nic->ext;
	iobase		= ext->iobase;

}

/*==============================================================================
 *
 */
void
macphy_event_handler( Nic_t *nic )
{
	struct _pulse		pulse;
	iov_t				iov;
	int					rcvid;
	Nic_Macphy_Ext_t	*ext;

	ext = (Nic_Macphy_Ext_t *)nic->ext;
	SETIOV( &iov, &pulse, sizeof( pulse ) );

	while( 1 ) {
        if( ( rcvid = MsgReceivev( ext->chid, &iov, 1, NULL ) ) == -1 ) {
            if( errno == ESRCH ) {
                pthread_exit( NULL );
            }
            continue;
        }

        pthread_mutex_lock( &ext->mutex );
		switch( pulse.code ) {
			case NIC_INTERRUPT_EVENT:
				macphy_process_interrupt( nic );
				InterruptUnmask( nic->cfg.IRQRegisters[0], ext->iid );
				break;

			case NIC_TIMER_EVENT:
				macphy_timer( nic ); 
				break;

			case MDI_TIMER:
				MDI_MonitorPhy( ext->mdi );
				break;

			default:
				if( rcvid ) {
					MsgReplyv( rcvid, ENOTSUP, &iov, 1 );
				}
				break;
		}
        pthread_mutex_unlock( &ext->mutex );
	}
}
