//--------------------------------------------------------------------
//  transmit.c
//
//  Copyright (c) 2005 National Semiconductor Corporation.
//  All Rights Reserved.
//
//  QNX6 Network Device Driver for National Semiconductor
//  DP83816 "MacPhyter" 10/100 Ethernet Controller.
//
//  io-net send_packets entry point and MPL transmit_complete callback.
//--------------------------------------------------------------------

#include <macphy.h>

/*******************************************************************************
 *
 */
int
macphy_send( Nic_t *nic )
{
	Nic_Macphy_Ext_t	*ext;
	int					iobase;

	ext		= (Nic_Macphy_Ext_t *)nic->ext;
	iobase	= ext->iobase;

}


/*******************************************************************************
 *
 * macphy_send_packets
 *
 * Description: Entry point from io-net layer by protocol to send
 *              packets out the NIC
 *
 */
int
macphy_send_packets(npkt_t *npkt, void *hdl)
{
	Nic_t				*nic = (Nic_t *)hdl;
	Nic_Macphy_Ext_t	*ext;

    net_buf_t  *buf;
    io_net_msg_join_mcast_t *msg;

	ext = (Nic_Macphy_Ext_t *)nic->ext;

    if( npkt->flags & _NPKT_MSG )
    {
        buf = TAILQ_FIRST( &npkt->buffers );
        if( buf != NULL )
        {
            /* when selective mutlicasting worked on this card 
               this is where it would happen */
            msg = (io_net_msg_join_mcast_t *)buf->net_iov->iov_base;
            if( msg )
            {
                switch( msg->type )
                {
                    case _IO_NET_JOIN_MCAST:
                        break;

                    case _IO_NET_REMOVE_MCAST:
                        break;

                    default:
                        if( ext->verbose )
                        {
                            slogf (_SLOGC_NETWORK, _SLOG_ERROR, "devn-macphy: "
                                     "Message type not supported.");
                        }
                        break;
                }
            }
        }

		ion_tx_complete(ext->reg_hdl, npkt);
    }
    else
    {
		/*  make sure the link is UP ...... */
		if( !ext->acceptingTx )
		{
			errno = ENOLINK;
			npkt->flags |= _NPKT_NOT_TXED;
        	ion_tx_complete( ext->reg_hdl, npkt );
			return (TX_DOWN_FAILED);
		}

		pthread_mutex_lock( &ext->mutex );

		if( ext->ntail )
		{
			ext->ntail->next = npkt;
		}
		else
		{
			ext->nhead = npkt;
		}
		while( npkt->next )
		{
			npkt = npkt->next;
		}
		ext->ntail = npkt;

		macphy_send( nic );
		pthread_mutex_unlock( &ext->mutex );

    }

    return (TX_DOWN_OK);
}


/*******************************************************************************
 *
 *
 */
int
macphy_tx_int_handler( Nic_t *nic )
{
}
