//*****************************************************************************
//
//  OAI.H
//
//  Copyright (c) 2004 National Semiconductor Corporation.
//  All Rights Reserved
//
//  This header file contains OS-specific services, data-type definitions,
//  etc. required by MPL. This file must be customized for each OS/Platform
//  and re-named OAI.H.  A version of this file will typically be created 
//  for each platform and exist in the NSM driver's source directory.
//
//  This version is for QNX6/x86 PC BSP.
//*****************************************************************************

#ifndef _OAI_H_
#define _OAI_H_

// XXX temporary until mplclient.h is done
#define IN
#define OUT
typedef struct _mpl_dmaregion_hndl {
    void *blah;
    void *blahtoo;
}MPL_DMAREGION_HNDL;
// XXX end fixups

//******************************** MPL General Options ************************
// Modify the following to set the desire MPL options
//*****************************************************************************

// On Big endian machines, uncomment the following line or define it via
// the compiler command line
//#define MPL_BIG_ENDIAN

// On machines that support 64-bit physical addressing, uncomment the following 
// line or define it via the compiler command line.
//#define MPL_64BIT_ADDR

//**************************** END OF MPL General Options *********************

//******************************** MPL Task Offload ***************************
// Modify the following to define MPL task offload behavior
//*****************************************************************************

// Uncomment to enable statistics collection task
//#define MPL_TASK_STAT

// Uncomment to enable VLAN offload task
//#define MPL_TASK_VLAN

// Uncomment to enable multicast filtering task
//#define MPL_TASK_MCAST

// Uncomment to enable checksum offload task
//#define MPL_TASK_STAT

//**************************** END OF MPL Task Offload Options ****************


//******************** MPL COMPILER INDEPENDENT DATA TYPES ********************
// Modify the following to define MPL data types in terms of native
// compiler data types.
//*****************************************************************************

// Note: On platforms where the natural integer size is less then 32-bits
// in size (eg 16-bit platforms), NS_UINT and NS_SINT must be defined as a 
// data type no less then 32-bits in size.
typedef uint32_t            NS_UINT;
typedef int32_t             NS_SINT;

// Fixed width data types/pointers  
typedef unsigned char       NS_CHAR;
typedef uint8_t             NS_UINT8;
typedef int8_t              NS_SINT8;
typedef uint16_t            NS_UINT16;
typedef int16_t             NS_SINT16;
typedef uint32_t            NS_UINT32;
typedef int32_t             NS_SINT32;

// Pointer precision data type - The MPL design requires the width of the
// following data type equal the size of native address pointers (32 or 64 bits).
// Note: On platforms that use address sizes less then 32-bits (16 or 24), 
// NS_ADDR must be defined as a 32-bit type.
#if defined(MPL_64BIT_ADDR)
typedef unsigned __int64    NS_ADDR;
#else
typedef unsigned int        NS_ADDR;
#endif

//**************** END OF MPL COMPILER INDEPENDENT DATA TYPES *****************

// Defines the maximum number of physical adapters supported by MPL.
// Customize this value appropriate to the environment.
#define MPL_MAX_ADAPTERS        8


//******************* OAI DATA DECLARATIONS (DO NOT MODIFY) *******************

// Void types
typedef void                NS_VOID;

// Boolean data type
typedef NS_UINT             NS_BOOLEAN;
#define NS_TRUE             1
#define NS_FALSE            0

// Handle structure for returning module opaque fields
typedef NS_UINT     MPL_HANDLE;

// timer callback function pointer (see OaixxxTimer functions)
typedef NS_VOID (MPL_TIMERCALLBACK)(NS_VOID *pTimerArg);

// handle for DMA region (see OaiAlloc/FreeDmaRegion)
typedef struct _MPL_MEM_REGION {
    NS_VOID *pAddr;    // aligned (as requested in alloc request) region ptr, logical
    NS_ADDR  phyAddr;  // aligned (as requested in alloc request) region ptr, physical
} MPL_MEM_REGION;


//************************** OAI FUNCTION PROTOTYPES **************************
// The following are function declarations of OS services required by MPL.
// Each function must be implemented in using native OS API's and linked
// with MPL
//*****************************************************************************

#if defined(__cplusplus)
extern "C"
{
#endif

//+++++ PCI Memory Mapped I/O access Functions

//Read from PCI memory-mapped IO location 'pAddr'.

#define OaiIoRead32( pClientDevHndl, pAddr) in32(pAddr)
#define OaiIoRead16( pClientDevHndl, pAddr) in16(pAddr)
#define OaiIoRead8( pClientDevHndl, pAddr)  in8(pAddr)

//Write to PCI memory-mapped IO location 'pAddr'.
NS_VOID
OaiIoWrite32(
    IN NS_VOID            *pClientDevHndl,
    IN NS_UINT32 volatile *pAddr,
    IN NS_UINT32           writeData
    );
NS_VOID
OaiIoWrite16(
    IN NS_VOID            *pClientDevHndl,
    IN NS_UINT16 volatile *pAddr,
    IN NS_UINT16           writeData
    );
NS_VOID
OaiIoWrite8(
    IN NS_VOID           *pClientDevHndl,
    IN NS_UINT8 volatile *pAddr,
    IN NS_UINT8          writeData
    );

//**************************************************************//
// 64-bit read/write from/to PCI memory-mapped IO               //
// location 'pAddr'.                                            //
// (NOTE: THE FOLLOWING NEED NOT BE IMPLEMENTED IN SYSTEMS THAT //
// DO NOT SUPPORT 64-BIT ADDRESSING)                            //
//**************************************************************//
#if defined(MPL_64BIT_ADDR)
NS_DATA64
OaiIoRead64(
    IN NS_VOID            *pClientDevHndl,
    IN NS_DATA64 volatile *pAddr
    );
NS_VOID
OaiIoWrite64(
    IN NS_VOID            *pClientDevHndl,
    IN NS_DATA64 volatile *pAddr,
    IN NS_DATA64           writeData
    );
#endif


//+++++ Host Memory Functions

//Allocate/free a region in host memory suitable for DMA
NS_BOOLEAN
OaiAllocDmaRegion(
    IN NS_VOID          *pClientDevHndl,
    IN NS_UINT           size,
    IN NS_UINT           byteAlignment,
    OUT MPL_MEM_REGION **pRgnHndl
    );

NS_VOID
OaiFreeDmaRegion(
    IN NS_VOID            *pClientDevHndl,
    IN MPL_DMAREGION_HNDL *pRgnHndl
    );

//Allocate/free general purpose host memory
NS_VOID *
OaiMalloc(
    IN NS_UINT regionSize
    );

NS_VOID
OaiFree(
    IN NS_UINT  regionSize,
    IN NS_VOID *startAddrLogical
    );

//Copy/zero host memory
NS_VOID
OaiMemCopy(
    IN NS_VOID *dest,
    IN NS_VOID *src,
    IN NS_UINT  len
    );

NS_VOID
OaiZeroMem(
    IN NS_VOID *pMemRegion,
    IN NS_UINT  regionSize
    );

//+++++ Timer Functions

//Delay program execution for 'sleepInterval' usec
NS_VOID
OaiSleep(
    IN NS_UINT sleepInterval
    );

//Return 10ms tick count
NS_UINT32   
OaiGetTickCount(
    );

//Allocate/free/start/cancel periodic timer w/ callback 'pTimerFunction'
NS_BOOLEAN
OaiCreateTimer( 
    IN MPL_TIMERCALLBACK  pTimerFunction,
    IN NS_VOID           *pTimerArg,
    OUT NS_VOID         **pTimerHandle
    );

NS_VOID
OaiDeleteTimer(
    IN NS_VOID *pTimerHandle
    );

NS_VOID 
OaiStartTimer(
    IN NS_VOID *pTimerHandle,
    IN NS_UINT  delayInMsec
    );

NS_BOOLEAN
OaiCancelTimer(
    IN NS_VOID *pTimerHandle
    );

//+++++ Resource Synchronization Functions

//Initialize/free/acquire/release multiprocessor-safe synchronization lock
NS_BOOLEAN
OaiCreateLock(
    OUT NS_VOID **pLockHandle
    );

NS_VOID
OaiDestroyLock(
    IN NS_VOID *pLockHandle
    );

NS_VOID
OaiAcquireLock(
    IN NS_VOID *pLockHandle
    );

NS_VOID
OaiReleaseLock(
    IN NS_VOID *pLockHandle
    );

#if defined(__cplusplus)
}
#endif 


//+++++ Debug Output Abstractions - Define appropriate to the environment.

// OAI_DEBUG_MSG takes a base string parameter, and zero or more optional 
// insertion parameters.
#define OAI_DEBUG_MSG define_a_debug_trace_function_here

// OAI_DEBUG_BREAK breaks into the debugger.
#define OAI_DEBUG_BREAK define_a_debug_break_function_here


//********************** END OF OAI FUNCTION PROTOTYPES ***********************


#endif // _OAI_H_
